package app.revanced.patches.kakaotalk.send

import app.revanced.patcher.Fingerprint
import app.revanced.patcher.extensions.InstructionExtensions.addInstructions
import app.revanced.patcher.extensions.InstructionExtensions.instructions
import app.revanced.patcher.extensions.InstructionExtensions.removeInstruction
import app.revanced.patches.kakaotalk.send.fingerprints.allowSwipeReplyToFeedFingerprint
import app.revanced.patches.kakaotalk.send.fingerprints.realActionForReplyFingerprint
import com.android.tools.smali.dexlib2.Opcode
import com.android.tools.smali.dexlib2.iface.instruction.formats.Instruction21t

@Suppress("unused")
val allowReplyToFeedPatch = app.revanced.patcher.patch.bytecodePatch(
    name = "Allow reply to feed",
    description = "Allows replying to feed messages",
) {
    compatibleWith("com.kakao.talk"("25.4.2"))

    execute {
        val patch: (Fingerprint, String, Boolean) -> Unit = { fp, register, bool ->
            val method = fp.method
            val insns = method.instructions

            val idxIfnez = insns.indexOfFirst { it is Instruction21t && it.opcode == Opcode.IF_NEZ }

            val idxInvoke = insns.subList(0, idxIfnez)
                .indexOfLast { it.opcode == Opcode.INVOKE_VIRTUAL }

            (idxInvoke until idxIfnez).toList()
                .sortedDescending()
                .forEach { method.removeInstruction(it) }

            method.addInstructions(
                idxInvoke,
                """
                    const/4 $register, ${if (bool) "0x1" else "0x0"}
                """.trimIndent()
            )
        }

        patch(realActionForReplyFingerprint, "v0", false)
        patch(allowSwipeReplyToFeedFingerprint, "p2", false)
    }
}